/*
 * MUST (Marmot Umpire Scalable Tool)
 * Copyright (C) ZIH, Technische Universitaet Dresden, Federal Republic of Germany, 2011-2011
 * Copyright (C) Lawrence Livermore National Laboratories, United States of America, 2011-2011
 *
 * See the file LICENSE.txt in the package base directory for details
 */

/**
 * @file BlockingOp.cpp
 *       @see must::BlockingOp.
 *
 *  @date 08.08.2011
 *  @author Tobias Hilbrich, Mathias Korepkat, Joachim Protze
 */

#include "BlockingOp.h"
#include "BlockingState.h"

using namespace must;

//=============================
// Constructor
//=============================
BlockingOp::BlockingOp (
        BlockingState* state,
        MustParallelId pId,
        MustLocationId lId
)
: myState (state),
  myPId (pId),
  myLId (lId)
{
    myRank = myState->myPIdMod->getInfoForId(pId).rank;
}

//=============================
// Destructor
//=============================
BlockingOp::~BlockingOp (void)
{
    myState = NULL;
}

//=============================
// getIssuerRank
//=============================
int BlockingOp::getIssuerRank (void)
{
    return myRank;
}

//=============================
// needsSecondary
//=============================
bool BlockingOp::needsSecondary (void)
{
    return false;
}

//=============================
// getPId
//=============================
MustParallelId BlockingOp::getPId (void)
{
    return myPId;
}

//=============================
// getLId
//=============================
MustLocationId BlockingOp::getLId (void)
{
    return myLId;
}

//=============================
// isMatchingColl
//=============================
bool BlockingOp::isMatchingColl (MustCollCommType collId, I_Comm *comm)
{
    //Per default false
    return false;
}

//=============================
// registerSecondaryP2P
//=============================
void BlockingOp::registerSecondaryOp (BlockingOp* secondary)
{
    //Nothing to do
}

//=============================
// applyRequestToWait
//=============================
bool BlockingOp::applyP2PToWait (
        std::string label,
        P2PInfo *info,
        std::list<int> *outToRanks,
        std::list<std::string> *outLabels,
        std::list<std::pair<bool, std::pair<MustParallelId,MustLocationId> > > *outReferences)
{
    if (!info->isWc)
    {
        outToRanks->push_back(info->target);
        if (outLabels) outLabels->push_back(label);
        if (outReferences) outReferences->push_back(std::make_pair(true, std::make_pair(info->pId, info->lId)));
    }
    else
    {
        I_Comm *comm = info->comm;
        I_GroupTable *table = comm->getGroup();

        //For intercomms we have to use the remote group!
        if (comm->isIntercomm())
            table = comm->getRemoteGroup();

        for (int i = 0; i < table->getSize(); i++)
        {
            int wRank;
            table->translate(i, &wRank);

            outToRanks->push_back(wRank);

            //We only add label and ref to the first arc to avoid cluttering
//            if (i == 0)
//            {
                if (outLabels) outLabels->push_back (label);
                if (outReferences) outReferences->push_back(std::make_pair(true, std::make_pair(info->pId, info->lId)));
//            }
//            else
//            {
//                if (outLabels) outLabels->push_back ("");
//                if (outReferences) outReferences->push_back(std::make_pair(false, std::make_pair(0,0)));
//            }
        }
    }

    return true;
}

/*EOF*/
