/*
 * MUST (Marmot Umpire Scalable Tool)
 * Copyright (C) ZIH, Technische Universitaet Dresden, Federal Republic of Germany, 2011-2011
 * Copyright (C) Lawrence Livermore National Laboratories, United States of America, 2011-2011
 *
 * See the file LICENSE.txt in the package base directory for details
 */

/**
 * @file I_Operation.h
 *       @see MUST::I_Operation.
 *
 *  @date 26.07.2011
 *  @author Tobias Hilbrich, Joachim Protze, Mathias Korepkat
 */

#include "GtiEnums.h"
#include <iostream>

#ifndef I_OPERATION_H
#define I_OPERATION_H

using namespace gti;

namespace must
{
    /**
     * Return state when processing an operation:
     * - PROCESSING_SUCCESS: successful processing, the
     *       operation can be removed from the queue
     * - PROCESSING_ERROR: some critical error occurred
     * - PROCESSING_REEXECUTE: the operation discovered
     *       that it can't be processed at the moment (and
     *       updated I_OperationReordering accordingly). This
     *       causes the operation to stay in the queues and be
     *       processed once its newly set block or suspension
     *       is removed.
     */
    enum PROCESSING_RETURN
    {
        PROCESSING_SUCCESS = 0,
        PROCESSING_ERROR,
        PROCESSING_REEXECUTE
    };

    /**
     * Interface for all operations that can be reordered.
     */
    class I_Operation
    {
    public:
        /**
         * Called when a operation should be processed.
         * @param rank to which the operation belongs.
         * @return GTI_SUCCESS if successful.
         */
        virtual PROCESSING_RETURN process (int rank) = 0;

        /**
         * Called when human readable information on
         * an operation shall be printed. Primary use case
         * is for debugging.
         * @param out stream to print into.
         * @return GTI_SUCCESS if successful.
         */
        virtual GTI_RETURN print (std::ostream &out) = 0;
    };
} /*namespace must*/

#endif /*I_OPERATION_H*/
