/*
 * MUST (Marmot Umpire Scalable Tool)
 * Copyright (C) ZIH, Technische Universitaet Dresden, Federal Republic of Germany, 2011-2011
 * Copyright (C) Lawrence Livermore National Laboratories, United States of America, 2011-2011
 *
 * See the file LICENSE.txt in the package base directory for details
 */

/**
 * @file I_ParallelIdAnalysis.h
 *       Interface for the location module.
 *
 *  @date 07.01.2011
 *  @author Tobias Hilbrich
 */

#include "I_Module.h"
#include "GtiEnums.h"

#include "BaseIds.h"

#ifndef I_PARALLELIDANALYSIS_H
#define I_PARALLELIDANALYSIS_H

/**
 * Information for a parallel id.
 */
struct ParallelInfo
{
	int rank; /**< Rank of the process.*/
};

/**
 * Parallel id analysis interface.
 */
class I_ParallelIdAnalysis : public gti::I_Module
{
public:
    /**
     * Returns information on the given id.
     * @param id to get parallel identifier information for.
     * @return information for this parallel id.
     */
    virtual ParallelInfo getInfoForId (MustParallelId id) = 0;

    /**
     * Returns a textual description of the parallel id.
     * @param id parallel id.
     * @return textual representation.
     */
    virtual std::string toString (MustParallelId id) = 0;

}; /*class I_ParallelIdAnalysis*/

#endif /*I_PARALLELIDANALYSIS_H*/
