/*
 * MUST (Marmot Umpire Scalable Tool)
 * Copyright (C) ZIH, Technische Universitaet Dresden, Federal Republic of Germany, 2011-2011
 * Copyright (C) Lawrence Livermore National Laboratories, United States of America, 2011-2011
 *
 * See the file LICENSE.txt in the package base directory for details
 */

/**
 * @file MsgLoggerStdOut.cpp
 *       @see MUST::MsgLoggerStdOut.
 *
 *  @date 20.01.2011
 *  @author Tobias Hilbrich
 */

#include "GtiMacros.h"

#include "MsgLoggerStdOut.h"

using namespace must;

mGET_INSTANCE_FUNCTION(MsgLoggerStdOut);
mFREE_INSTANCE_FUNCTION(MsgLoggerStdOut);
mPNMPI_REGISTRATIONPOINT_FUNCTION(MsgLoggerStdOut);

//=============================
// Constructor
//=============================
MsgLoggerStdOut::MsgLoggerStdOut (const char* instanceName)
    : gti::ModuleBase<MsgLoggerStdOut, I_MessageLogger> (instanceName),
      myPIdModule (NULL),
      myLIdModule (NULL)
{
    //create sub modules
    std::vector<I_Module*> subModInstances;
    subModInstances = createSubModuleInstances ();

    //save sub modules
    myLIdModule = (I_LocationAnalysis*) subModInstances[0];
    myPIdModule = (I_ParallelIdAnalysis*) subModInstances[1];
}

//=============================
// Destructor
//=============================
MsgLoggerStdOut::~MsgLoggerStdOut (void)
{
	if (myLIdModule)
		destroySubModuleInstance ((I_Module*) myLIdModule);
	myLIdModule = NULL;

	if (myPIdModule)
		destroySubModuleInstance ((I_Module*) myPIdModule);
	myPIdModule = NULL;
}

//=============================
// log
//=============================
GTI_ANALYSIS_RETURN MsgLoggerStdOut::log (
					int msgId,
					int hasLocation,
					uint64_t pId,
					uint64_t lId,
					int msgType,
					char *text,
					int textLen,
					int numReferences,
					uint64_t* refPIds,
					uint64_t* refLIds)
{

	switch ((MustMessageType)msgType)
	{
	case MustErrorMessage:
		std::cout << "Error";
		break;
	case MustWarningMessage:
		std::cout << "Warning";
		break;
	case MustInformationMessage:
		std::cout << "Information";
		break;
	default:
		std::cout << "Unknown";
	}

	if (hasLocation)
		std::cout
			<< ": from: "
			<< myLIdModule->toString(pId, lId)
			<< "@"
			<< myPIdModule->toString(pId)
			<< ": ";
	else
		std::cout << " global: ";

	std::cout << text;

	std::cout
		<< " References: ";

	for (int i = 0; i < numReferences; i++)
	{
		std::cout
				<< "Reference "<<(i+1)<<": "
				<< myLIdModule->toString(refPIds[i], refLIds[i])
				<< "@"
				<< myPIdModule->toString(refPIds[i])
				<< "; ";
	}

	std::cout
		<< std::endl;

	if ((MustMessageType)msgType == MustErrorMessage)
		std::cout.flush();

	return GTI_ANALYSIS_SUCCESS;
}

/*EOF*/
