/*
 * MUST (Marmot Umpire Scalable Tool)
 * Copyright (C) ZIH, Technische Universitaet Dresden, Federal Republic of Germany, 2011-2011
 * Copyright (C) Lawrence Livermore National Laboratories, United States of America, 2011-2011
 *
 * See the file LICENSE.txt in the package base directory for details
 */

/**
 * @file CommPredefs.cpp
 *       @see MUST::CommPredefs
 *
 *  @date 04.03.2011
 *  @author Mathias Korepkat, Tobias Hilbrich
 */

#include "GtiMacros.h"
#include "MustEnums.h"
#include "ResourceApi.h"
#include "mustFeaturetested.h"
#include "mustConfig.h"

#include "CommPredefs.h"

#include <mpi.h>
#include <pnmpimod.h>

using namespace must;

mGET_INSTANCE_FUNCTION(CommPredefs);
mFREE_INSTANCE_FUNCTION(CommPredefs);
mPNMPI_REGISTRATIONPOINT_FUNCTION(CommPredefs);

//=============================
// Constructor
//=============================
CommPredefs::CommPredefs (const char* instanceName)
    : gti::ModuleBase<CommPredefs, I_CommPredefs> (instanceName)
{
    //create sub modules
    std::vector<I_Module*> subModInstances;
    subModInstances = createSubModuleInstances ();

    //handle sub modules
    if (subModInstances.size() > 0)
    {
    		for (int i = 0; i < subModInstances.size(); i++)
    			destroySubModuleInstance (subModInstances[i]);
    }

    //No sub modules needed

    //Initialize module data
    //Nothing to do
}

//=============================
// Destructor
//=============================
CommPredefs::~CommPredefs ()
{
	//nothing to do
}

//=============================
// propagate
//=============================
GTI_ANALYSIS_RETURN CommPredefs::propagate ()
{
	//==Call the API call to propagate the communicator information to I_CommTrack
	propagateCommsP fP;
	if (getWrapperFunction ("propagateComms", (GTI_Fct_t**)&fP) == GTI_SUCCESS)
	{
		int rank, size;
		int err;
		PNMPI_modHandle_t handle;
		PNMPI_Service_descriptor_t service;
		PNMPI_Service_Fct_t fct;
		MPI_Comm world = MPI_COMM_WORLD;

		//We need to check whether MPI_COMM_WORLD was splited
#ifdef PNMPI_FIXED
		err = PNMPI_Service_GetModuleByName("split_processes", &handle);
#else
		char string[512];
		sprintf (string, "%s","split_processes");
		err = PNMPI_Service_GetModuleByName(string, &handle);
#endif
		if (err == PNMPI_SUCCESS)
		{
			MPI_Comm thisSetComm;

			err = PNMPI_Service_GetServiceByName(handle, "SplitMod_getMySetComm", "p", &service);
			assert (err == PNMPI_SUCCESS);
			((int(*)(void*)) service.fct) (&thisSetComm);

			world = thisSetComm;
			PMPI_Comm_rank (thisSetComm, &rank);
			PMPI_Comm_size (thisSetComm, &size);
		}
		else
		{
			//No splitting is active, use MPI_COMM_WORLD
			PMPI_Comm_rank (MPI_COMM_WORLD, &rank);
			PMPI_Comm_size (MPI_COMM_WORLD, &size);
		}

		int worldF = MUST_Comm_c2f (world);

		(*fP) (
			rank,
			rank,
			size,
			MUST_Comm_c2f (MPI_COMM_NULL),
			MUST_Comm_c2f (MPI_COMM_SELF),
			MUST_Comm_c2f (MPI_COMM_WORLD), //we must use the real MPI_COMM_WORLD here
			1,
			&worldF //we must use the possibly replaced MPI_COMM_WORLD here
			);
	}

	return GTI_ANALYSIS_SUCCESS;
}

/*EOF*/
