/*
 * MUST (Marmot Umpire Scalable Tool)
 * Copyright (C) ZIH, Technische Universitaet Dresden, Federal Republic of Germany, 2011-2011
 * Copyright (C) Lawrence Livermore National Laboratories, United States of America, 2011-2011
 *
 * See the file LICENSE.txt in the package base directory for details
 */

/**
 * @file DatatypeTrackDerivedStorage.h
 *       @see MUST::DatatypeTrack.
 *
 *  @date 10.02.2011
 *  @author Tobias Hilbrich, Joachim Protze
 */

#include "ModuleBase.h"
#include "I_DatatypeTrack.h"
#include "TrackBase.h"
#include "Datatype.h"

#include <map>
#include <string.h>
#include <utility>

#ifndef DATATYPETRACKDERIVEDSTORAGE_H
#define DATATYPETRACKDERIVEDSTORAGE_H

#define MUST_ADDR_INFTY (((MustAddressType)1 << (sizeof(MustAddressType)*8-1)) ^ -1)
#define MUST_ADDR_NEG_INFTY (((MustAddressType)1 << (sizeof(MustAddressType)*8-1)))

using namespace gti;

namespace must
{
    /**
     * Information for a contiguous type.
     */
    class FullContiguousTypeInfo : public Datatype
    {
        public:
            int count; 
            MustDatatypeType parentType;
            std::list<MustDatatypeType> getReferencedTypes(); /**< @see MUST::Datatype::getReferencedTypes.*/
            FullContiguousTypeInfo(
                    DatatypeTrack *track,
                    MustParallelId pId,
                    MustLocationId lId,
                    int count,
                    MustDatatypeType oldType,
                    std::vector<Datatype *> oldInfos
            );
            bool printRealDatatypePos(std::ostream &out, MustAddressType errorpos);
            bool printRealDatatypeLongPos(std::ostream &out, MustAddressType errorpos);
        protected:
            MustTypemapType getFullTypemap(int* err);
            void getRealBlockInfo ();
            void getRealTypesig(MustTypesig * typesig, int * err);
            void getRealTypesig(MustTypesigType * typesig, int * err);
/*            template<class T>
            void getRealTypesig(T* typesig, int* err);*/
    };

    /**
     * Information for a vector type.
     */
    class FullVectorTypeInfo : public Datatype
    {
        public:
            int count;
            int blocklength;
            int stride;
            MustDatatypeType parentType;
            std::list<MustDatatypeType> getReferencedTypes(); /**< @see MUST::Datatype::getReferencedTypes.*/
            FullVectorTypeInfo(
                    DatatypeTrack *track,
                    MustParallelId pId,
                    MustLocationId lId,
                    int count,
                    int blocklength,
                    int stride,
                    MustDatatypeType oldType,
                    std::vector<Datatype *> oldInfos
            );
            bool printRealDatatypePos(std::ostream &out, MustAddressType errorpos);
            bool printRealDatatypeLongPos(std::ostream &out, MustAddressType errorpos);
        protected:
            MustTypemapType getFullTypemap(int* err);
            void getRealBlockInfo ();
            void getRealTypesig(MustTypesig * typesig, int * err);
            void getRealTypesig(MustTypesigType * typesig, int * err);
/*            template<class T>
            void getRealTypesig(T* typesig, int* err);*/
    };

    /**
     * Information for a hvector type.
     */
    class FullHVectorTypeInfo : public Datatype
    {
        public:
            int count;
            int blocklength;
            MustAddressType stride;
            MustDatatypeType parentType;
            std::list<MustDatatypeType> getReferencedTypes(); /**< @see MUST::Datatype::getReferencedTypes.*/
            FullHVectorTypeInfo(
                    DatatypeTrack *track,
                    MustParallelId pId,
                    MustLocationId lId,
                    int count,
                    int blocklength,
                    MustAddressType stride,
                    MustDatatypeType oldType,
                    std::vector<Datatype *> oldInfos
            );
            bool printRealDatatypePos(std::ostream &out, MustAddressType errorpos);
            bool printRealDatatypeLongPos(std::ostream &out, MustAddressType errorpos);
        protected:
            MustTypemapType getFullTypemap(int* err);
            void getRealBlockInfo ();
            void getRealTypesig(MustTypesig * typesig, int * err);
            void getRealTypesig(MustTypesigType * typesig, int * err);
/*            template<class T>
            void getRealTypesig(T* typesig, int* err);*/
    };

    /**
     * Information for a indexed type.
     */
    class FullIndexedTypeInfo : public Datatype
    {
        public:
            int count;
            int* arrayOfBlocklengths;
            int* arrayOfDisplacements;
            MustDatatypeType parentType;
            ~FullIndexedTypeInfo(void); /**< Destructor.*/
            FullIndexedTypeInfo(void); /**< Constructor.*/
            FullIndexedTypeInfo(FullIndexedTypeInfo& info);  /**< Copy constructor.*/
            std::list<MustDatatypeType> getReferencedTypes(); /**< @see MUST::Datatype::getReferencedTypes.*/
            FullIndexedTypeInfo(
                    DatatypeTrack *track,
                    MustParallelId pId,
                    MustLocationId lId,
                    int count,
                    int* blocklengths,
                    int* displacements,
                    MustDatatypeType oldType,
                    std::vector<Datatype *> oldInfos
            );
            bool printRealDatatypePos(std::ostream &out, MustAddressType errorpos);
            bool printRealDatatypeLongPos(std::ostream &out, MustAddressType errorpos);
        protected:
            MustTypemapType getFullTypemap(int* err);
            void getRealBlockInfo ();
            void getRealTypesig(MustTypesig * typesig, int * err);
            void getRealTypesig(MustTypesigType * typesig, int * err);
/*            template<class T>
            void getRealTypesig(T* typesig, int* err);*/
    };

    /**
     * Information for a hindexed type.
     */
    class FullHIndexedTypeInfo : public Datatype
    {
        public:
            int count;
            int* arrayOfBlocklengths;
            MustAddressType* arrayOfDisplacements;
            MustDatatypeType parentType;
            ~FullHIndexedTypeInfo(void); /**< Destructor.*/
            FullHIndexedTypeInfo(void); /**< Constructor.*/
            FullHIndexedTypeInfo(FullHIndexedTypeInfo& info);  /**< Copy constructor.*/
            std::list<MustDatatypeType> getReferencedTypes(); /**< @see MUST::Datatype::getReferencedTypes.*/
            FullHIndexedTypeInfo(
                    DatatypeTrack *track,
                    MustParallelId pId,
                    MustLocationId lId,
                    int count,
                    int* blocklengths,
                    MustAddressType* displacements,
                    MustDatatypeType oldType,
                    std::vector<Datatype *> oldInfos
            );
            bool printRealDatatypePos(std::ostream &out, MustAddressType errorpos);
            bool printRealDatatypeLongPos(std::ostream &out, MustAddressType errorpos);
        protected:
            MustTypemapType getFullTypemap(int* err);
            void getRealBlockInfo ();
            void getRealTypesig(MustTypesig * typesig, int * err);
            void getRealTypesig(MustTypesigType * typesig, int * err);
/*            template<class T>
            void getRealTypesig(T* typesig, int* err);*/
    };

    /**
     * Information for a struct type.
     */
    class FullStructTypeInfo : public Datatype
    {
        public:
            int count;
            int* arrayOfBlocklengths;
            MustAddressType* arrayOfDisplacements;
            MustDatatypeType* parentTypes;
            ~FullStructTypeInfo(void); /**< Destructor.*/
            FullStructTypeInfo(void); /**< Constructor.*/
            FullStructTypeInfo(FullStructTypeInfo& info); /**< Copy constructor.*/
            std::list<MustDatatypeType> getReferencedTypes(); /**< @see MUST::Datatype::getReferencedTypes.*/
            FullStructTypeInfo(
                    DatatypeTrack *track,
                    MustParallelId pId,
                    MustLocationId lId,
                    int count,
                    int* blocklengths,
                    MustAddressType* displacements,
                    MustDatatypeType* oldTypes,
                    std::vector<Datatype *> oldInfos
            );
            bool printRealDatatypePos(std::ostream &out, MustAddressType errorpos);
            bool printRealDatatypeLongPos(std::ostream &out, MustAddressType errorpos);
        protected:
            MustTypemapType getFullTypemap(int* err);
            void getRealBlockInfo ();
            void getRealTypesig(MustTypesig * typesig, int * err);
            void getRealTypesig(MustTypesigType * typesig, int * err);
/*            template<class T>
            void getRealTypesig(T* typesig, int* err);*/
    };

    /**
     * Information for a indexed-block type.
     */
    class FullIndexedBlockTypeInfo : public Datatype
    {
        public:
            int count;
            int blocklength;
            int* arrayOfDisplacements;
            MustDatatypeType parentType;
            ~FullIndexedBlockTypeInfo(void); /**< Destructor.*/
            FullIndexedBlockTypeInfo(void); /**< Constructor.*/
            FullIndexedBlockTypeInfo(FullIndexedBlockTypeInfo& info); /**< Copy Constructor.*/
            std::list<MustDatatypeType> getReferencedTypes(); /**< @see MUST::Datatype::getReferencedTypes.*/
            FullIndexedBlockTypeInfo(
                    DatatypeTrack *track,
                    MustParallelId pId,
                    MustLocationId lId,
                    int count,
                    int blocklength,
                    int* displacements,
                    MustDatatypeType oldType,
                    std::vector<Datatype *> oldInfos
            );
            bool printRealDatatypePos(std::ostream &out, MustAddressType errorpos);
            bool printRealDatatypeLongPos(std::ostream &out, MustAddressType errorpos);
        protected:
            MustTypemapType getFullTypemap(int* err);
            void getRealBlockInfo ();
            void getRealTypesig(MustTypesig * typesig, int * err);
            void getRealTypesig(MustTypesigType * typesig, int * err);
/*            template<class T>
            void getRealTypesig(T* typesig, int* err);*/
    };

    /**
     * Information for a reduced type.
     */
    class FullResizedTypeInfo : public Datatype
    {
        public:
            MustDatatypeType parentType;
            std::list<MustDatatypeType> getReferencedTypes(); /**< @see MUST::Datatype::getReferencedTypes.*/
            FullResizedTypeInfo(
                    DatatypeTrack *track,
                    MustParallelId pId,
                    MustLocationId lId,
                    MustAddressType lb,
                    MustAddressType extent,
                    MustDatatypeType oldType,
                    std::vector<Datatype *> oldInfos
            );
            bool printRealDatatypePos(std::ostream &out, MustAddressType errorpos);
            bool printRealDatatypeLongPos(std::ostream &out, MustAddressType errorpos);
        protected:
            MustTypemapType getFullTypemap(int* err);
            void getRealBlockInfo ();
            void getRealTypesig(MustTypesig * typesig, int * err);
            void getRealTypesig(MustTypesigType * typesig, int * err);
/*            template<class T>
            void getRealTypesig(T* typesig, int* err);*/
    };

    /**
     * Information for a subarray type.
     */
    class FullSubarrayTypeInfo : public Datatype
    {
        public:
            int ndims;
            int* arrayOfSizes;
            int* arrayOfSubsizes;
            int* arrayOfStarts;
            int order;
            MustDatatypeType parentType;
            ~FullSubarrayTypeInfo(void); /**< Destructor.*/
            FullSubarrayTypeInfo(void); /**< Constructor.*/
            FullSubarrayTypeInfo(FullSubarrayTypeInfo& info); /**< Copy Constructor.*/
            std::list<MustDatatypeType> getReferencedTypes(); /**< @see MUST::Datatype::getReferencedTypes.*/
            FullSubarrayTypeInfo(
                    DatatypeTrack *track,
                    MustParallelId pId,
                    MustLocationId lId,
                    int ndims,
                    int* sizes,
                    int* subsizes,
                    int* starts,
                    int order,
                    MustDatatypeType oldType,
                    std::vector<Datatype *> oldInfos
            );
            bool printRealDatatypePos(std::ostream &out, MustAddressType errorpos);
            bool printRealDatatypeLongPos(std::ostream &out, MustAddressType errorpos);
        protected:
            MustTypemapType getFullTypemap(int* err);
            void getRealBlockInfo ();
            void getRealTypesig(MustTypesig * typesig, int * err);
            void getRealTypesig(MustTypesigType * typesig, int * err);
/*            template<class T>
            void getRealTypesig(T* typesig, int* err);*/
    };

    /**
     * Information for a darray type.
     */
    class FullDarrayTypeInfo : public Datatype
    {
        public:
            int commSize;
            int rank;
            int ndims;
            int* arrayOfGsizes;
            int* arrayOfDistribs;
            int* arrayOfDargs;
            int* arrayOfPsizes;
            int order;
            MustDatatypeType parentType;
            ~FullDarrayTypeInfo(void); /**< Destructor.*/
            FullDarrayTypeInfo(void); /**< Constructor.*/
            FullDarrayTypeInfo(FullDarrayTypeInfo& info); /**< Copy Constructor.*/
            std::list<MustDatatypeType> getReferencedTypes(); /**< @see MUST::Datatype::getReferencedTypes.*/
            FullDarrayTypeInfo(
                    DatatypeTrack *track,
                    MustParallelId pId,
                    MustLocationId lId,
                    int commSize,
                    int rank,
                    int ndims,
                    int* gsizes,
                    int* distribs,
                    int* dargs,
                    int* psizes,
                    int order,
                    MustDatatypeType oldType,
                    std::vector<Datatype *> oldInfos
            );
            bool printRealDatatypePos(std::ostream &out, MustAddressType errorpos);
            bool printRealDatatypeLongPos(std::ostream &out, MustAddressType errorpos);
        protected:
            MustTypemapType getFullTypemap(int* err);
            void getRealBlockInfo ();
            void getRealTypesig(MustTypesig * typesig, int * err);
            void getRealTypesig(MustTypesigType * typesig, int * err);
/*            template<class T>
            void getRealTypesig(T* typesig, int* err);*/
    };

//     #include "DatatypeTrackDerivedStorage.hpp"

} /*namespace MUST*/

#endif /*DATATYPETRACKDERIVEDSTORAGE_H*/
