/*
 * MUST (Marmot Umpire Scalable Tool)
 * Copyright (C) ZIH, Technische Universitaet Dresden, Federal Republic of Germany, 2010-2011
 * Copyright (C) Lawrence Livermore National Laboratories, United States of America, 2010-2011
 *
 * See the file LICENSE.txt in the package base directory for details
 */

/**
 * @file RequestTrack.h
 *       @see MUST::RequestTrack.
 *
 *  @date 24.01.2010
 *  @author Tobias Hilbrich
 */

#include "ModuleBase.h"
#include "I_RequestTrack.h"
#include "TrackBase.h"
#include "Request.h"
#include "I_DatatypeTrack.h"
#include "I_CommTrack.h"
#include "I_BaseConstants.h"

#ifndef REQUESTTRACK_H
#define REQUESTTRACK_H

using namespace gti;

namespace must
{
	/**
	 * Enumeration of all predefined requests (none).
	 */
	enum MustMpiRequestPredefined
	{
		MUST_MPI_REQUEST_UNKNOWN = 0
	};

	/**
     * Implementation of I_RequestTrack.
     */
    class RequestTrack : public TrackBase<Request, MustRequestType, MustMpiRequestPredefined, RequestTrack, I_RequestTrack>
    {
    public:
        /**
         * Constructor.
         * @param instanceName name of this module instance.
         */
    		RequestTrack (const char* instanceName);

    		/**
    		 * Destructor.
    		 */
    		virtual ~RequestTrack (void);

    		/**
    		 * @see I_RequestTrack::createPersistentSend
    		 */
    		GTI_ANALYSIS_RETURN createPersistentSend (
    				MustParallelId pId,
    				MustLocationId lId,
    				int count,
    				MustDatatypeType datatype,
    				int dest,
    				int tag,
    				MustCommType comm,
    				int sendMode,
    				MustRequestType request);

    		/**
    		 * @see I_RequestTrack::createPersistentRecv
    		 */
    		GTI_ANALYSIS_RETURN createPersistentRecv (
    				MustParallelId pId,
    				MustLocationId lId,
    				int count,
    				MustDatatypeType datatype,
    				int source,
    				int tag,
    				MustCommType comm,
    				MustRequestType request);

    		/**
    		 * @see I_RequestTrack::cancel
    		 */
    		GTI_ANALYSIS_RETURN cancel (
    				MustParallelId pId,
    				MustLocationId lId,
    				MustRequestType request);

    		/**
    		 * @see I_RequestTrack::addActive
    		 */
    		GTI_ANALYSIS_RETURN addActive (
    				MustParallelId pId,
    				MustLocationId lId,
    				int isSend,
    				MustRequestType request,
    	            int destSource);

    		/**
    		 * @see I_RequestTrack::forceFree
    		 */
    		GTI_ANALYSIS_RETURN forceFree (
    				MustParallelId pId,
    				MustLocationId lId,
    				MustRequestType request);

    		/**
    		 * @see I_RequestTrack::startPersistent
    		 */
    		GTI_ANALYSIS_RETURN startPersistent (
    				MustParallelId pId,
    				MustLocationId lId,
    				MustRequestType request);

    		/**
    		 * @see I_RequestTrack::startPersistentArray
    		 */
    		GTI_ANALYSIS_RETURN startPersistentArray (
    				MustParallelId pId,
    				MustLocationId lId,
    				MustRequestType* requests,
    				int count);

    		/**
    		 * @see I_RequestTrack::complete
    		 */
    		GTI_ANALYSIS_RETURN complete (
    				MustParallelId pId,
    				MustLocationId lId,
    				MustRequestType request,
    				int flag);

    		/**
    		 * @see I_RequestTrack::completeAny
    		 */
    		GTI_ANALYSIS_RETURN completeAny (
    					MustParallelId pId,
    					MustLocationId lId,
    					MustRequestType* requests,
    					int count,
    					int index,
    					int flag);

    		/**
    		 * @see I_RequestTrack::completeArray
    		 */
    		GTI_ANALYSIS_RETURN completeArray (
    				MustParallelId pId,
    				MustLocationId lId,
    				MustRequestType *requests,
    				int count,
    				int flag);

    		/**
    		 * @see I_RequestTrack::completeSome
    		 */
    		GTI_ANALYSIS_RETURN completeSome (
    				MustParallelId pId,
    				MustLocationId lId,
    				MustRequestType *requests,
    				int count,
    				int *indices,
    				int numIndices);

    		/**
    		 * @see I_RequestTrack::getRequest
    		 */
    		I_Request* getRequest (
    		        MustParallelId pId,
    		        MustRequestType request);

    		/**
    		 * @see I_RequestTrack::getRequest
    		 */
    		I_Request* getRequest (
    		        int rank,
    		        MustRequestType comm);

    		/**
    		 * @see I_RequestTrack::getPersistentRequest
    		 */
    		I_RequestPersistent* getPersistentRequest (
    		        MustParallelId pId,
    		        MustRequestType request);

    		/**
    		 * @see I_RequestTrack::getPersistentRequest
    		 */
    		I_RequestPersistent* getPersistentRequest (
    		        int rank,
    		        MustRequestType comm);

    protected:

    		I_DatatypeTrack* myDTrack;
    		I_CommTrack* myCTrack;
    		I_BaseConstants* myConsts;

    		/**
    		 * Implementation of TrackBase::createPredefinedInfo.
    		 */
    		Request* createPredefinedInfo (int value, MustRequestType handle);

    }; /*class RequestTrack */
} /*namespace MUST*/

#endif /*RequestTrack_H*/
