/*
 * MUST (Marmot Umpire Scalable Tool)
 * Copyright (C) ZIH, Technische Universitaet Dresden, Federal Republic of Germany, 2011-2011
 * Copyright (C) Lawrence Livermore National Laboratories, United States of America, 2011-2011
 *
 * See the file LICENSE.txt in the package base directory for details
 */

/**
 * @file cfortran.cpp
 * This is a a test for the analysis group BasicChecks.
 *
 * Description:
 * Performs a MPI_Send and calls recv_, what is a fortran subroutine that calls MPI_Recv.
 *
 *  @date 16.08.2011
 *  @author Mathias Korepkat
 */
#include <iostream>
#include <mpi.h>

extern "C" void myrecv_ (int* out);

int main (int argc, char** argv)
{
    int rank,size,out;

    MPI_Init(&argc,&argv);
    MPI_Comm_rank (MPI_COMM_WORLD, &rank);
    MPI_Comm_size (MPI_COMM_WORLD, &size);

	//Enough tasks ?
	if (size < 2)
	{
		std::cerr << "This test needs at least 2 processes!"<< std::endl;
		MPI_Finalize();
		return 1;
	}

    std::cout << "Hello, I am "<<rank<<" of "<<size<<" tasks.\n"<<std::endl;

	if (rank == 0)
	{
		MPI_Send (&size, 1, MPI_INT, 1, 42, MPI_COMM_WORLD);
		std::cout <<"rank 0 send: "<<size<<std::endl;
	}

	out = -1;
	if (rank == 1)
	{
		myrecv_ (&out);
		std::cout <<"rank 1 recived: "<<out<<std::endl;
	}

    std::cout << "Signing off, rank " << rank << "." << std::endl;
    MPI_Finalize ();

    return 0;
}
