/*
 * MUST (Marmot Umpire Scalable Tool)
 * Copyright (C) ZIH, Technische Universitaet Dresden, Federal Republic of Germany, 2011-2011
 * Copyright (C) Lawrence Livermore National Laboratories, United States of America, 2011-2011
 *
 * See the file LICENSE.txt in the package base directory for details
 */

/**
 * @file MatchContiMissmatchError.c
 * Type matching test with an error.
 *
 * Description:
 * A single send-recv match, the send uses a contiguous type the receive not, the spanned typesignatues
 * do not match (MPI_CHAR != MPI_UNSIGNED_CHAR) (Error).
 *
 * @author Tobias Hilbrich
 */

#include <mpi.h>
#include <stdio.h>

int main (int argc, char** argv)
{
    int rank,size;
    char data[10];
    unsigned char udata[10];
    MPI_Status status;
    MPI_Datatype conti, uconti;

    MPI_Init(&argc,&argv);
    MPI_Comm_rank (MPI_COMM_WORLD, &rank);
    MPI_Comm_size (MPI_COMM_WORLD, &size);

    //Enough tasks ?
    	if (size < 2)
    	{
    		printf ("This test needs at least 2 processes!\n");
    		MPI_Finalize();
    		return 1;
    	}

    	//Say hello
    	printf ("Hello, I am rank %d of %d processes.\n", rank, size);


    	if (rank == 0)
        {
                //Create a conti type
                MPI_Type_contiguous (10, MPI_CHAR, &conti);
                MPI_Type_commit (&conti);
                //and another conti type
                MPI_Type_contiguous (10, MPI_UNSIGNED_CHAR, &uconti);
                MPI_Type_commit (&uconti);
    		MPI_Send (data, 1, conti, 1, 666, MPI_COMM_WORLD);
        }

    	if (rank == 1)
        {
                //Create a conti type
                MPI_Type_contiguous (10, MPI_UNSIGNED_CHAR, &uconti);
                MPI_Type_commit (&uconti);
                //and another conti type
                MPI_Type_contiguous (10, MPI_CHAR, &conti);
                MPI_Type_commit (&conti);
    		MPI_Recv (udata, 1, conti, 0, 666, MPI_COMM_WORLD, &status);
        }

        MPI_Type_free(&conti);
        MPI_Type_free(&uconti);
    	//Say bye bye
    	printf ("Signing off, rank %d.\n", rank);

    MPI_Finalize ();

    return 0;
}
