#! @BASH@
#
# MUST (Marmot Umpire Scalable Tool)
# Copyright (C) ZIH, Technische Universitaet Dresden, Federal Republic of Germany, 2011-2011
# Copyright (C) Lawrence Livermore National Laboratories, United States of America, 2011-2011
#
# See the file LICENSE.txt in the package base directory for details
#
#@@
# mustrun - tool to run applications with the MPI correctenss tool MUST
#

#----------------- Function ----------------
#name: printHelp
#purpose: prints the help for this script
#-------------------------------------------
printHelp()
{
    echo "\"mustrun\" from MUST v@MUST_VERSION@"
    echo "Prepares and runs an application that is linked with P^nMPI for runtime analysis with the MPI correctness tool MUST."
	echo ""
	echo "Replace your regular mpiexec/mpirun command with mustrun, usage: "
	echo "mustrun [--help|-help|--must:help] [--must:nocrash] [--must:layout <LAYOUT.xml>]" 
	echo "        [--must:mode {prepare|run|preparerun}] [--must:np <NP-SWITCH>]" 
	echo "        [--must:mpiexec <RUNCOMMAND>] [--must:temp <DIR>]" 
	echo "        [--must:analyses <ANALYSES-XMLS>] [--must:apis <API-XMLS>]"
	echo "        [--must:verbose] [--must:quiet]" 
	echo "        <MPIRUNARGS> <COMMAND> <ARGS>"
    echo "--help,-help,--must:help: Prints this information."
    echo "--must:nocrash: When using the default layout, switches on asynchronous "
	echo "                communication to improve tool performance, only use this option" 
	echo "                if your applications suffers from no crash."
    echo "--must:layout <LAYOUT.xml>: Overrides the default MUST layout with the given" 
	echo "                            layout, must be an absolute file-path."
    echo "--must:language {c|fortran}: Specifies in which prgramming language your program"
    echo "                             is written. This is needed to use the right "
    echo "                             library of pnmpi. Use \"c\" for C++ applications. If nothing is set," 
    echo "                             the correct language will be detected automatically."
    echo "--must:mode {prepare|run|preparerun}: Operation mode of mustrun, \"prepare\"" 
	echo "                                      only creates the intermediate files needed" 
	echo "                                      by MUST, \"run\" only runs an application" 
	echo "                                      that was previously prepared with mustrun,"
	echo "                                      \"preparerun\" performs a \"prepare\"" 
	echo "                                      first and a \"run\" afterwards, default is"
	echo "                                      \"preparerun\"."
	echo "--must:np <NP-SWITCH>: Tells MUST to search for <NP-SWITCH> in <MPIRUNARGS>"
	echo "                       for determining the number of application processes,"
	echo "                       example: \"--must:np -np\"."
	echo "--must:mpiexec <RUNCOMMAND>: Tells MUST to use the given command instead of "
	echo "                             mpiexec."
	echo "--must:temp <DIR>: Specifies which directory should be used to store" 
	echo "                   intermediate files, or in which directory the preparation"
	echo "                   of this run was made, default is \"must_temp\"."
	echo "--must:analyses <ANALYSES-XMLS>: Gives extra analysis specifications to the"
	echo "                                 weaver, multiple xmls need to be seperated"
	echo "                                 with a column."
	echo "--must:apis <API-XMLS>: Gives extra api specifications to the weaver,"
	echo "                        multiple xmls need to be seperated with a column."
    echo "--must:verbose: Prints extensive information, use this when submitting bug" 
	echo "                reports." 
	echo "--must:timing: Adds a \"time\" to the actual mpirun call, used to discern"
	echo "               mustrun overheads from the actual runtime." 
	echo "--must:quiet: Causes mustrun to give no output at all (if successful)."
    echo "<MPIRUNARGS>: Arguments you ususally pass to mpirun/mpiexec, mustrun searches" 
	echo "              for a \"-n\" \"-np\" \"-p\" in the given order to determine how" 
	echo "              many tasks should be used for the application."
    echo "<COMMAND>: Application to execute."
    echo "<ARGS>: Application arguments." 
	echo ""
	echo "Note that the default layout used for MUST requires one additional process," 
	echo "mustrun will execute an mpiexec with the number of application processes you" 
	echo "specified plus 1, make sure to allocate the sufficent resources."
	echo ""
	echo "Example: mustrun -np 4 myapp.exe"
	echo "(Prepares and runs \"maypp.exe\" with default layout)"
	echo ""
	echo "Contact: tobias.hilbrich@tu-dresden.de"
}

#Exit if something goes wrong
set -e

#####################################################################
## 1) clean up the input (remove double spaces,replace tabs by spaces,
##    remove leading spaces)
input=`echo "$*" | sed -e 's/[	]/ /g' -e 's/  / /g' -e 's/^[ ]*//g'`

#####################################################################
## 2) is the input empty ?? Or is --help, -help, or --must:help present ?? 
##    If so print Help !
printhelp=0

if [ "x$input" == "x" ]
then printhelp=1
fi

temp=`echo " "$input" " | @AWK@ -- '{ind=index ($0," --help ")}{if (ind != 0) print "HELP"}'`
if [ "x$temp" == "xHELP" ]
then printhelp=1
fi

temp=`echo " "$input" " | @AWK@ -- '{ind=index ($0," -help ")}{if (ind != 0) print "HELP"}'`
if [ "x$temp" == "xHELP" ]
then printhelp=1
fi

temp=`echo " "$input" " | @AWK@ -- '{ind=index ($0," --must:help ")}{if (ind != 0) print "HELP"}'`
if [ "x$temp" == "xHELP" ]
then printhelp=1
fi

if [ "x$printhelp" == "x1" ]
then 
    printHelp
    exit $?
fi  

#####################################################################
## 2a) Is "--must:verbose" present ?? 
##    If so be verbose
verbose=`echo "$input" | @AWK@ -- '{ind=index ($0,"--must:verbose")}{if (ind != 0) print "1"}'`
temp=`echo "$input" | sed -e s/--must:verbose//`
input=$temp

## 2b) Is "--must:quiet" present ?? 
##    If so be quiet
quiet=`echo "$input" | @AWK@ -- '{ind=index ($0,"--must:quiet")}{if (ind != 0) print "1"}'`
temp=`echo "$input" | sed -e s/--must:quiet//`
input=$temp

## 2c) Is "--must:timing" present ?? 
##    If so do timing
timing=`echo "$input" | @AWK@ -- '{ind=index ($0,"--must:timing")}{if (ind != 0) print "1"}'`
temp=`echo "$input" | sed -e s/--must:timing//`
input=$temp

if [ "x$verbose" == "x1" ]
then
	echo "CTEST_FULL_OUTPUT" # this output is necessary to get the full output of tests
							 # when sending them to the dashboard. 

	echo "========= Step 1) Setup ========="
fi

#####################################################################
## 3) Which mode are we using (--must:mode)
modeRun=1
modePrepare=1
mode=`echo "$input" | @AWK@ -- '{ind=index ($0,"--must:mode")}{if (ind != 0) print "1"}'`
if [ "x$mode" == "x1" ]
then
	mode=`echo "$input" | sed -e "s/.*--must:mode[[:space:]]\([^[:space:]]*\).*/\1/"`
	if [ "x$mode" == "xprepare" ]
	then
		modeRun=0
		modePrepare=1
	else
		if [ "x$mode" == "xrun" ]
		then
			modeRun=1
			modePrepare=0
		else
			if [ "x$mode" == "xpreparerun" ]
			then
				modeRun=1
				modePrepare=1
			else
				echo "Unknown mode given, valid are \"prepare\", \"run\", and \"preparerun\"."
				exit 1
			fi
		fi
	fi
fi
temp=`echo "$input" | sed -e "s/--must:mode[[:space:]][^[:space:]]*//"`
input=$temp

#Print result
if [ "x$verbose" == "x1" ]
then
	echo -n "Mode: "
	if [ "x$modePrepare" == "x1" ]
	then
		echo -n "Prepare "
	fi
	if [ "x$modeRun" == "x1" ]
	then
		echo -n "Run"
	fi
	echo ""
fi

#####################################################################
## 4) Determine specified number of processes
switches="-np -n -p"
found=0
override=`echo "$input" | sed -e "s/.*--must:np[[:space:]]\([^[:space:]]*\).*/\1/"`

if [ "x$override" != "x$input" ]
then
	switches=$override
	#User specified --must:np, remove it from the input
	temp=`echo "$input" | sed -e "s/--must:np[[:space:]][^[:space:]]*//"`
	input=$temp
fi

for switch in $switches
do
	pTemp=`echo "$input" | sed -e "s/.*$switch[[:space:]]\([^[:space:]]*\).*/\1/"`
	
	if [ "x$found" == "x0" ]
	then
		if [ "x$pTemp" != "x$input" ]
		then
			p=$pTemp
			found=1
			switchToUse=$switch
		fi
	fi
done

#Did we find the number of tasks
if [ "x$found" == "x0" ]
then
	echo "Could not determine number of MPI tasks, use \"--must:np <NP-SWITCH>\" to tip me."
	exit 1
fi

#Print result
if [ "x$verbose" == "x1" ]
then
	echo "Application will use "$p" tasks."
fi

#####################################################################
## 4a) Determine whether timeout should be set
found=0
timeout=0
override=`echo "$input" | sed -e "s/.*--must:timeout[[:space:]]\([^[:space:]]*\).*/\1/"`

if [ "x$override" != "x$input" ]
then
    timeout=$override
    #User specified --must:timeout, remove it from the input
    temp=`echo "$input" | sed -e "s/--must:timeout[[:space:]][^[:space:]]*//"`
    input=$temp
fi

#Print result
if [ "x$verbose" == "x1" ]
then
    echo "Application will use a timeout of "$timeout" seconds."
fi



#####################################################################
## 5) Determine a temp directory
tempDir=`echo "$input" | sed -e "s/.*--must:temp[[:space:]]\([^[:space:]]*\).*/\1/"`

if [ "x$tempDir" == "x$input" ]
then
	tempDir="must_temp"
else
	temp=`echo "$input" | sed -e "s/--must:temp[[:space:]][^[:space:]]*//"`
	input=$temp
	
	#make relative pathes absolute
	temp=`echo "$tempDir" | sed -e "s%^[/].*%%"`		
	if [ "x$temp" != "x" ]
	then
		tempDir="$PWD/$tempDir"
	fi
fi

if [ -d $tempDir ]
then
	if [ "x$quiet" != "x1" ]
	then
		echo "Information: overwritting old intermediate data in directory \""$tempDir"\"!"
	fi
else
	mkdir $tempDir
fi

#Print result
if [ "x$verbose" == "x1" ]
then
	echo "Using intermediate directory \""$tempDir"\"."
fi

#####################################################################
## 6) What layout are we using (--must:layout)
layoutFile=""
printLayoutFile=""
if [ "x$modePrepare" == "x1" ]
then
	userLayout=`echo "$input" | @AWK@ -- '{ind=index ($0,"--must:layout")}{if (ind != 0) print "1"}'`

	if [ "x$userLayout" == "x1" ]
	then
	    ## a) user specified layout
		layoutFile=`echo "$input" | sed -e "s/.*--must:layout[[:space:]]\([^[:space:]]*\).*/\1/"`
		
		#make relative pathes absolute
		temp=`echo "$layoutFile" | sed -e "s%^[/].*%%"`		
		if [ "x$temp" != "x" ]
		then
			layoutFile="$PWD/$layoutFile"
		fi
		
		if [ -f "$layoutFile" ]
		then
		    echo -n ""
		else
			echo "Can't find the given layout file \""$layoutFile"\"!"
			exit 1
		fi	
		
		printLayoutFile="$layoutFile"
		temp=`echo "$input" | sed -e "s/--must:layout[[:space:]][^[:space:]]*//"`
		input=$temp
	else
		## b) Default layout
		nocrash=`echo "$input" | @AWK@ -- '{ind=index ($0,"--must:nocrash")}{if (ind != 0) print "1"}'`
		temp=`echo "$input" | sed -e s/--must:nocrash//`
		input=$temp
		
		layoutFile=layout.xml
		printLayoutFile=$tempDir/$layoutFile
		
		strategy="CStratSimpleUp"
		place="mpi_place"
		
		if [ "x$nocrash" == "x1" ]
		then
			strategy="CStratThreadedUp"
		fi
			
		cat <<End-of-File > $printLayoutFile
<layout-specification>
	<levels>
		<level order="0" size="$p" place-name="">
			<analyses>
			    <analysis name="DatatypePredefs" group="MUST_Resource"/>
			    <analysis name="CommPredefs" group="MUST_Resource"/>
			    <analysis name="FinalizeNotify" group="MUST_Base"/>
			    <analysis name="OpSplitter" group="MUST_Deadlock"/>
			    <analysis name="WcUpdate" group="MUST_Deadlock"/>
				<analysis name="IntegerChecks" group="MUST_BasicChecks"/>
				<analysis name="BasicIntegrities" group="MUST_BasicChecks"/>
				<analysis name="OperationChecks" group="MUST_BasicChecks"/>
				<analysis name="CommChecks" group="MUST_CommChecks"/>
                <analysis name="RequestChecks" group="MUST_RequestChecks"/>
                <analysis name="DatatypeChecks" group="MUST_DatatypeChecks"/>
                <analysis name="OverlapChecks" group="MUST_OverlapChecks"/>
                <analysis name="GroupChecks" group="MUST_GroupChecks"/>
                <analysis name="LeakChecks" group="MUST_LeakChecks"/>
                <analysis name="RequestCondition" group="MUST_RequestCondition"/>
                <analysis name="CollectiveCondition" group="MUST_CollectiveCondition"/>
                <analysis name="OnlyOnRootCondition" group="MUST_OnlyOnRootCondition"/>
				<!--Add new local checks here-->
			</analyses>
		</level>
		<level order="1" size="1" place-name="$place">
			<analyses>
				<analysis name="BlockingState" group="MUST_Deadlock"/>			
				<analysis name="MsgLoggerHtml" group="MUST_Base"/>
				<!--Add new non-local checks here-->
			</analyses>
		</level>
	</levels>
	<communications>
		<default>
			<comm-strategy name="$strategy">
				<settings></settings>
			</comm-strategy>
			<comm-protocol name="CProtMpiSplited">
				<settings></settings>
			</comm-protocol>
		</default>
		<connection top-level="1" bottom-level="0"></connection>
	</communications>
</layout-specification>		
End-of-File
	fi

	#Print result
    if [ "x$verbose" == "x1" ]
	then
		echo "Using layout file: "$printLayoutFile
	fi
fi

#Get rid of arguments we might not have used
temp=`echo "$input" | sed -e s/--must:nocrash//`
input=$temp
temp=`echo "$input" | sed -e "s/--must:layout[[:space:]][^[:space:]]*//"`
input=$temp

#####################################################################
## 6b) Are there extra analysis or API specifications
analyses=`echo "$input" | sed -e "s/.*--must:analyses[[:space:]]\([^[:space:]]*\).*/\1/"`
if [ "x$analyses" == "x$input" ]
then
	analyses=""
else
	#remove from the line
	temp=`echo "$input" | sed -e "s/--must:analyses[[:space:]][^[:space:]]*//"`
	input=$temp
	
	#replace the semicoloumns by spaces
	analyses=`echo "$analyses" | sed -e "s/:/ /g"`
	
	#make relative pathes absolute
	analysesTemp=""
	for a in $analyses
	do
		temp=`echo "$a" | sed -e "s%^/.*%%"`		
		if [ "x$temp" == "x" ]
		then
			analysesTemp="$analysesTemp $a"
		else
			analysesTemp="$analysesTemp $PWD/$a"			
		fi
	done
	analyses=$analysesTemp
fi

apis=`echo "$input" | sed -e "s/.*--must:apis[[:space:]]\([^[:space:]]*\).*/\1/"`
if [ "x$apis" == "x$input" ]
then
	apis=""
else
	#remove from the line
	temp=`echo "$input" | sed -e "s/--must:apis[[:space:]][^[:space:]]*//"`
	input=$temp
	
	#replace the semicoloumns by spaces
	apis=`echo "$apis" | sed -e "s/:/ /g"`
	
	#make relative pathes absolute
	apisTemp=""
	for a in $apis
	do
		temp=`echo "$a" | sed -e "s%^[/].*%%"`		
		if [ "x$temp" == "x" ]
		then
			apisTemp="$analysesTemp $a"
		else			
			apisTemp="$analysesTemp $PWD/$a"
		fi
	done
	apis=$apisTemp
fi

#####################################################################
## 7) Determine the mpiexec command
mpiexec=`echo "$input" | sed -e "s/.*--must:mpiexec[[:space:]]\([^[:space:]]*\).*/\1/"`

if [ "x$mpiexec" == "x$input" ]
then
	mpiexec="mpiexec"
else
	temp=`echo "$input" | sed -e "s/--must:mpiexec[[:space:]][^[:space:]]*//"`
	input=$temp
	export GTI_MPIEXEC=$mpiexec
fi

#Print result
if [ "x$verbose" == "x1" ]
then
	echo "Using mpiexec: "$mpiexec
fi

#####################################################################
## 8) Run the weaver and code generators

if [ "x$verbose" == "x1" ]
then
	echo "========= Step 2) Generation ========="
fi

if [ "x$modePrepare" == "x1" ]
then
	#change to temp dir
	oldDir=$PWD
	cd $tempDir
	
	#run the weaver
	set +e
	@WEAVER_EXE@ \
		$layoutFile \
		@GTI_SPECIFICATION@ \
        @CMAKE_INSTALL_PREFIX@/specifications/mpi_specification.xml \
        @CMAKE_INSTALL_PREFIX@/specifications/mpi_2_specification.xml \
		@CMAKE_INSTALL_PREFIX@/specifications/must_base_api_spec.xml \
		@CMAKE_INSTALL_PREFIX@/specifications/must_resource_track_api.xml \
		@CMAKE_INSTALL_PREFIX@/specifications/must_deadlock_detection_api.xml \
        @CMAKE_INSTALL_PREFIX@/specifications/request_condition_api.xml \
        @CMAKE_INSTALL_PREFIX@/specifications/collective_condition_api.xml \
		@CMAKE_INSTALL_PREFIX@/specifications/onlyOnRoot_condition_api.xml \
		$apis \
		@CMAKE_INSTALL_PREFIX@/specifications/mpi_base_specification.xml \
		@CMAKE_INSTALL_PREFIX@/specifications/must_base_specification.xml \
		@CMAKE_INSTALL_PREFIX@/specifications/must_resource_track.xml \
		@CMAKE_INSTALL_PREFIX@/specifications/must_basic_checks.xml \
		@CMAKE_INSTALL_PREFIX@/specifications/must_comm_checks.xml \
        @CMAKE_INSTALL_PREFIX@/specifications/must_request_checks.xml \
        @CMAKE_INSTALL_PREFIX@/specifications/must_datatype_checks.xml \
        @CMAKE_INSTALL_PREFIX@/specifications/must_overlap_checks.xml \
        @CMAKE_INSTALL_PREFIX@/specifications/must_group_checks.xml \
		@CMAKE_INSTALL_PREFIX@/specifications/must_leak_checks.xml \
        @CMAKE_INSTALL_PREFIX@/specifications/request_condition.xml \
        @CMAKE_INSTALL_PREFIX@/specifications/collective_condition.xml \
		@CMAKE_INSTALL_PREFIX@/specifications/must_deadlock_detection.xml \
		@CMAKE_INSTALL_PREFIX@/specifications/onlyOnRoot_condition.xml \
		$analyses > output.out 2>output.err
		#TODO: add XMLS for further checks
	
	if [ "x$?" != "x0" ]
	then
		echo "Running the weaver failed with the following output:"
		cat output.out
		cat output.err
		exit 1
	fi
	
	if [ "x$verbose" == "x1" ]
	then
		echo "==> Running weaver:"
		cat output.out
		cat output.err
	else
		if [ "x$quiet" != "x1" ]
		then
			echo "Weaver ... success"
		fi
	fi
    
	#run the Generators
	export PATH=@GTI_HOME@/bin:$PATH
	bash weaver-run-generators.sh > output.out 2>output.err
	
	if [ "x$?" != "x0" ]
	then
		echo "Code generation failed with the following output:"
		cat output.out
		cat output.err
		exit 1
	fi
	
	if [ "x$verbose" == "x1" ]
	then
		echo "==> Generating source code:"
		cat output.out
		cat output.err
	else
		if [ "x$quiet" != "x1" ]
		then
			echo "Code generation ... success"
		fi
	fi

	#run the build file generation
	@BUILDGEN_EXE@ weaver-buildgen.xml > output.out 2>output.err
	
	if [ "x$?" != "x0" ]
	then
		echo "Build file generation failed with the following output:"
		cat output.out
		cat output.err
		exit 1
	fi
	
	if [ "x$verbose" == "x1" ]
	then
		echo "==> Generating build files:"
		cat output.out
		cat output.err
	else
		if [ "x$quiet" != "x1" ]
		then
			echo "Build file generation ... success"
		fi
	fi	
	set -e
fi

#####################################################################
## 9) Build the intermediate modules

if [ "x$modePrepare" == "x1" ]
then
	if [ -d BUILD ]
	then
		rm -rf BUILD
	fi

	mkdir BUILD
	cd BUILD
	
	set +e
	## Configure with cmake
	CC=@MUST_C_COMPILER@ CXX=@MUST_CXX_COMPILER@ FC=@MUST_Fortran_COMPILER@ cmake ../ -DCMAKE_BUILD_TYPE=@CMAKE_BUILD_TYPE@ -DENABLE_OTF_TRACING=@ENABLE_OTF_TRACING@ -DCMAKE_INSTALL_PREFIX=$PWD/install > output.out 2>output.err
		
	if [ "x$?" != "x0" ]
	then
		echo "Configuration of intermediate build failed with the following output:"
		cat output.out
		cat output.err
		exit 1
	fi
	
	if [ "x$verbose" == "x1" ]
	then
		echo "==> Configuring intermediate build:"
		cat output.out
		cat output.err
	else
		if [ "x$quiet" != "x1" ]
		then
			echo "Configuring intermediate build ... success"
		fi
	fi
	
    makej=8
    if [ $makej -gt $p ]
    then
        makej=$p
    fi

	## Make
    if [ "x$verbose" != "x1" -a "x$quiet" != "x1" ]
    then
        echo -n "Building intermediate sources ... "
    fi
	make -j${makej} > output.out 2>output.err
		
	if [ "x$?" != "x0" ]
	then
		echo "Building intermediate sources failed with the following output:"
		cat output.out
		cat output.err
		exit 1
	fi
	
	if [ "x$verbose" == "x1" ]
	then
		echo "==> Building intermediate sources:"
		cat output.out
		cat output.err
	else
		if [ "x$quiet" != "x1" ]
		then
			echo "success"
		fi
	fi
	
	## Make install
	make -j${makej} install > output.out 2>output.err
	
	if [ "x$?" != "x0" ]
	then
		echo "Installing intermediate modules failed with the following output:"
		cat output.out
		cat output.err
		exit 1
	fi
	
	if [ "x$verbose" == "x1" ]
	then
		echo "==> Installing intermediate modules:"
		cat output.out
		cat output.err
	else
		if [ "x$quiet" != "x1" ]
		then
			echo "Installing intermediate modules ... success"
		fi
	fi
	
	set -e
	cd ..
fi

#####################################################################
## 10) Create the P^nMPI configuration file

if [ "x$modePrepare" == "x1" ]
then
	## Get information on the layers we had in the layout
	numLayers=`grep "num-layers" weaver-layout-info.xml | sed -e s/.*num-layers=\"// -e s/[^0-9].*//`
	totalSize=0
	
	for (( i = 0 ; i < $numLayers ; i++ ))
	do
		size[$i]=`grep "id=\"$i\"" weaver-layout-info.xml | sed -e s"/.*size=\"//" -e s"/[^0-9].*//"`
		to[$i]=`grep "id=\"$i\"" weaver-layout-info.xml | sed -e s"/[^>]*[>]//" -e s"/[<][/]layer[>]//" -e s"/[<]to[>]/ /" -e s"/[<][/]to[>]/ /"`
		
		totalSize=$(($totalSize+${size[$i]}))
	done
	
	## Store the total size in an info file
	echo "$totalSize" > mustrun.info

	## Print the merge file
	cat <<End-of-File > level_merge.xml
<level-merge>
	<set>
End-of-File

	for (( i = 0 ; i < $numLayers ; i++ ))
	do
		echo -e "\t\t<order>$i</order>" >> level_merge.xml
	done
	
	cat <<End-of-File >> level_merge.xml
	</set>
</level-merge>
End-of-File

	## Run the module configurator
	set +e
	@MODCONFGEN_EXE@ weaver-mod-conf-input.xml level_merge.xml > output.out 2>output.err

	if [ "x$?" != "x0" ]
	then
		echo "Generating P^nMPI configuration failed:"
		cat output.out
		cat output.err
		exit 1
	fi
	
	if [ "x$verbose" == "x1" ]
	then
		echo "==> Generating P^nMPI configuration:"
		cat output.out
		cat output.err
	else
		if [ "x$quiet" != "x1" ]
		then
			echo "Generating P^nMPI configuration ... success"
		fi
	fi
	set -e

	##Add configuration for split module to the beginning of the config file
	if [ "@USE_CALLPATH@" == "ON" ]
	then
		cat <<End-of-File > pnmpi.conf
module libcallpathModule
End-of-File
	else
        echo -n "" > pnmpi.conf
	fi

	cat <<End-of-File >> pnmpi.conf
module libcProtMpiSplit
argument num_sets $numLayers
End-of-File
	
	confOut="weaver-mod-conf"
	for (( i = 0 ; i < $numLayers ; i++ ))
	do
		echo "argument size_$i ${size[$i]}" >> pnmpi.conf
		echo "argument stack_$i level_$i" >> pnmpi.conf
		
		confOut=$confOut.$i
	done

	index=0
	commId=1
	echo -n > pnmpi.conf_temp
	for (( i = 0 ; i < $numLayers ; i++ ))
	do
		for t in ${to[$i]}
		do
			echo "argument mapping$(($index)) $i:$commId:$t" >> pnmpi.conf_temp
			echo "argument mapping$(($index+1)) $t:$commId:$i" >> pnmpi.conf_temp
			index=$(($index+2))
			commId=$(($commId+1))
		done
	done

	echo "argument num_mappings $index" >> pnmpi.conf
	cat < pnmpi.conf_temp >> pnmpi.conf
	echo "" >> pnmpi.conf

	cat < $confOut >>pnmpi.conf
fi

#####################################################################
## 11) Create the module directory

if [ "x$modePrepare" == "x1" ]
then
	#Important: (TODO) PnMPI only supports one module dir, we create a new one here that unites 
	#                 the project modules, the intermediate modules and the GTI modules.
	if [ -d module_temp ]
	then
		rm -rf module_temp
	fi
	mkdir module_temp
	cp @GTI_MODULE_DIR@/* module_temp
	cp @CMAKE_INSTALL_PREFIX@/modules/* module_temp
	cp BUILD/install/modules/* module_temp
	
	##Go back to the right directory
	cd $oldDir
fi

#####################################################################
## 12) If we only run, get the total number of tasks we are going to use

if [ "x$modePrepare" == "x0" ]
then
	if [ -f $tempDir/mustrun.info ]
	then
		totalSize=`cat $tempDir/mustrun.info`
	else
		echo "Error: the \"mustrun.info\" file could not be found in the temp directory \"$tempDir\", did you really prepare with this temporary directory?"
		exit 1
	fi
fi

#####################################################################
## 13) Which programming language the program is using (--must:language)

langSet=0
myNM=@CMAKE_NM@
language=`echo "$input" | /usr/bin/awk -- '{ind=index ($0,"--must:language")}{if (ind != 0) print "1"}'`

if [ "x$language" == "x1" ]
then
    language=`echo "$input" | sed -e "s/.*--must:language[[:space:]]\([^[:space:]]*\).*/\1/"`
    if [ "x$language" == "xc" ]
    then
        langSet=1
    else
        if [ "x$language" == "xfortran" ]
        then
            langSet=1
        else
            echo "Unknown language given, valid are \"c\" and \"fortran\", use \"c\" for C++ appliccations."
            exit 1
        fi
    fi
fi

if [ "x$langSet" == "x0" ]
then
    executable=""
    language="c"
    if which $myNM >/dev/null; then

        for i in $input;
        do
            if [[ $i != -* && -x $i ]]
            then
                executable=$i
                # we have found an executable:detect language
                if test -z "$($myNM $executable | grep MAIN__)"
                then
                    language="c"
                else
                    language="fortran"
                fi    
            fi
        done
    fi
fi

temp=`echo "$input" | sed -e "s/--must:language[[:space:]][^[:space:]]*//"`
input=$temp

# #####################################################################
# ## 14) Do we want to ld_preload the pnmpi-lib?

if [ "x$quiet" != "x1" ]
then
    echo -n "Search for preloaded P^nMPI ... "
fi
executable=""
myldd=@LDD@
myotool=@OTOOL@
# search for the executable
for i in $input;
do
    if [[ $i != -* && -f $i && -x $i ]]
    then
        executable=$i
        # we have found an executable: is pnmpi-lib loaded already?
        havepnmpi=0
#         if  -x $executable
#         then
            if test -n "$myldd" -a -x "$myldd"
            then
                if test -z "$($myldd $executable | grep pnmpi)"
                then
                    if [ "x$quiet" != "x1" ]
                    then
                        echo -n "not found ... "
                    fi
                else
                    havepnmpi=1
                    if [ "x$quiet" != "x1" ]
                    then
                        echo -n "found ... "
                    fi
                fi
            elif test -n "$myotool" -a -x "$myotool"
            then
                if test -z "$($myotool -L $executable | grep pnmpi)"
                then
                    if [ "x$quiet" != "x1" ]
                    then
                        echo -n "not found ... "
                    fi
                else
                    havepnmpi=1
                    if [ "x$quiet" != "x1" ]
                    then
                        echo -n "found ... "
                    fi
                fi
            fi
#         fi
        break
    fi
done
# insert pnmpize if needed
if test ${havepnmpi} -eq 0
then
    temp=""
    found=0
    for i in $input;
    do
        # find the executable to inject pnmpize
        if test $found -eq 0 -a -x "$i"
        then
            # check for local executable
            if test "`basename $i`" == "$i"
            then
                temp="${temp} @BASH@ @CMAKE_INSTALL_PREFIX@/bin/pnmpize $language ./$i"
            else
                temp="${temp} @BASH@ @CMAKE_INSTALL_PREFIX@/bin/pnmpize $language $i"
            fi
            found=1
        else
            temp="${temp} $i"
        fi
    done
    input=${temp}
fi
if [ "x$quiet" != "x1" ]
then
    echo "success"
fi
#####################################################################
## 15) Run (finally :D)

if [ "x$timing" == "x1" ]
then
    # use 'time' to get the runtime of mpiexec
    mpiexec="time $mpiexec"
fi

if [ "x$modeRun" == "x1" ]                    
then     
	if [ "x$verbose" == "x1" ]
	then
		echo "========= Step 3) Execution ========="
	else
		if [ "x$quiet" != "x1" ]
		then
			echo "Executing application:"
		fi
		
		#Tell P^nMPI to be quiet
		export PNMPI_BE_SILENT=1
	fi

	set +e #we ignore the final exit status, this simplifies regular expressions for tests
	
	export PNMPI_LIB_PATH=$tempDir/module_temp
	export LD_LIBRARY_PATH=$tempDir/module_temp:@PnMPI_LIBRARY_DIR@:$LD_LIBRARY_PATH
	export DYLD_LIBRARY_PATH=$tempDir/module_temp:@PnMPI_LIBRARY_DIR@:$DYLD_LIBRARY_PATH
	export PNMPI_CONF=$tempDir/pnmpi.conf

	#Replace the given number of tasks with the real number
    
	temp=`echo "$input" | sed -e "s/$switchToUse[[:space:]]\([^[:space:]]*\)/$switchToUse $totalSize/"`
	input=$temp
	
	#Verbose
	if [ "x$verbose" == "x1" ]
	then
		echo "Going to execute run with:"
		echo "$mpiexec $input"
	fi
	
    if test $timeout -ne 0
    then
        bash -i -c "@CMAKE_INSTALL_PREFIX@/bin/timeout -t $timeout $mpiexec $input"
    else
        $mpiexec $input
    fi
fi
exit 0;
